<?php

if (!defined('WEBTOTEM_INIT') || WEBTOTEM_INIT !== true) {
	if (!headers_sent()) {
		header('HTTP/1.1 403 Forbidden');
	}
	die('Protected By WebTotem!');
}

class WebTotemAjax {

	/**
	 * Activation plugin.
	 *
	 * @return void
	 */
	public static function activation() {

		if (WebTotemRequest::post('ajax_action') !== 'activation') {
			return;
		}

		if($api_key = WebTotemRequest::post('api_key')) {

			$result = WebTotemAPI::auth($api_key);

			$response['notifications'] = self::notifications();
			if($result == 'success') {
				if(WebTotem::isMultiSite()) {
					$link = WebTotem::adminURL('admin.php?page=wtotem_all_sites');
				} else {
					$link = WebTotem::adminURL('admin.php?page=wtotem');
				}
				wp_send_json([
					'link' => $link,
					'success' => true,
					'user' => WebTotemAPI::getEmail(),
				], 200);
			} else {
				wp_send_json([
					'notifications' => self::notifications(),
					'success' => false,
				], 200);
			}
		}

	}

	/**
	 * The process of installing agents (WAF, AV) on the main page.
	 *
	 * @return void
	 */
	public static function agentsInstallation() {

		if (WebTotemRequest::post('ajax_action') !== 'agents_installation') {
			return;
		}

		$av_installed = WebTotemOption::getOption('av_installed');
		$waf_installed = WebTotemOption::getOption('waf_installed');

		// Check if the agents are installed.
		if ($av_installed and $waf_installed) {
			$agents_statuses = [
				'process_statuses' => [
					'av' => 'installed',
					'waf' => 'installed',
				],
			];
		}
		else {
			// If not installed, then request statuses from the WebTotem API.
			$host = WebTotemAPI::siteInfo();
			$data = WebTotemAPI::getAgentsStatusesFromAPI($host['id']);

			$agents_statuses = [
				'av' => $data['av']['status'],
				'waf' => $data['waf']['status'],
			];

			$agents_statuses = WebTotem::getAgentsStatuses($agents_statuses);
		}

		$build[] = [
			'variables' => [
				'process_status' => $agents_statuses['process_statuses'],
			],
			'template' => 'agents_installation',
		];

		$status = [
			'av' => $agents_statuses['process_statuses']['av'] == 'installed',
			'waf' => $agents_statuses['process_statuses']['waf'] == 'installed',
		];

		WebTotemOption::setOptions([
			'av_installed' => $status['av'],
			'waf_installed' => $status['waf'],
		]);

		$template = new WebTotemTemplate();
		$agents = $template->arrayRender($build);

		wp_send_json([
			'success' => true,
			'notifications' => self::notifications(),
			'agents' => $agents,
			'agents_statuses' => $status['av'] && $status['waf'],
		]);
	}


	/**
	 * Reinstall agents.
	 *
	 * @return void
	 */
	public static function reinstallAgents() {

		if (WebTotemRequest::post('ajax_action') !== 'reinstall_agents') {
			return;
		}

		if (WebTotemAgentManager::removeAgents()) {
			WebTotemAgentManager::amInstall();
		}
		$response['success'] = true;
		$response['redirect_link'] = WebTotem::adminURL('admin.php?page=wtotem');
		wp_send_json($response);

	}

	/**
	 * Deleting plugin activation data and redirecting to the activation page.
	 *
	 * @return void
	 */
	public static function logout() {

		if (WebTotemRequest::post('ajax_action') !== 'logout') {
			return;
		}

		WebTotemOption::logout();

		$response['success'] = true;
		$response['redirect_link'] = WebTotem::adminURL('admin.php?page=wtotem_activation');
		wp_send_json($response);

	}

	/**
	 * Creating a modal window.
	 *
	 * @return void
	 */
	public static function popup() {

		if (WebTotemRequest::post('ajax_action') !== 'popup') {
			return;
		}

		$action = WebTotemRequest::post('popup_action');
		$template = new WebTotemTemplate();

		if($action){
			switch ($action) {
				case 'reinstall_agents':
					$build[] = [
						'variables' => [
							'message' => sprintf(__('Some scanning data for %s may be deleted.', 'wtotem'), WEBTOTEM_SITE_DOMAIN),
							'action' => 'reinstall_agents',
							'page_nonce' => wp_create_nonce('wtotem_page_nonce'),
						],
						'template' => 'popup',
					];
					break;

				case 'logout':
					$build[] = [
						'variables' => [
							'message' => __('Are you sure you want to change the API key?', 'wtotem'),
							'action' => 'logout',
							'page_nonce' => wp_create_nonce('wtotem_page_nonce'),
						],
						'template' => 'popup',
					];
					break;
			}

			wp_send_json([
				'success' => true,
				'content' => $template->arrayRender($build),
			]);
		}

		wp_send_json([
			'success' => false,
		]);

	}

	/**
	 * Request to update charts with parameters.
	 *
	 * @return void
	 */
	public static function chart() {

		if (WebTotemRequest::post('ajax_action') !== 'chart') {
			return;
		}

		$template = new WebTotemTemplate();

		$days = (integer) WebTotemRequest::post('days');
		$service = WebTotemRequest::post('service');

		$host = WebTotemAPI::siteInfo();

		switch ($service) {
			case 'waf':

				WebTotemOption::setSessionOptions(['firewall_period' => $days]);

				// Firewall chart.
				$data = WebTotemAPI::getFirewallChart($host['id'], $days);
				$chart = WebTotem::generateWafChart($data['chart']);

				$_chart[] = [
					'variables' => [
						'days' => $days,
						'chart' => $chart['chart'],
					],
					'template' => 'firewall_chart',
				];

				// Firewall logs.
				$data = WebTotemAPI::getFirewall($host['id'], 10, NULL, $days);
				$firewall = $data['firewall'];

				$waf_logs[] = [
					'variables' => [
						'logs' => WebTotem::wafLogs($firewall['logs']['edges']),
					],
					'template' => 'firewall_logs',
				];

				// Firewall stats.
				$waf_stats[] = [
					'variables' => [
						'is_waf_training' => WebTotem::isWafTraining($data['agentManager']['createdAt']),
						'all_attacks' => $chart['count_attacks'],
						'blocking' => $chart['count_blocks'],
						'not_blocking' => $chart['count_attacks'] - $chart['count_blocks'],
						'most_attacks' => WebTotem::getMostAttacksData($firewall['map']),
					],
					'template' => 'firewall_stats',
				];

				WebTotemOption::setSessionOptions([
					'firewall_cursor' => $firewall['logs']['pageInfo']['endCursor'],
				]);

				$has_next_page = $firewall['logs']['pageInfo']['hasNextPage'];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'waf_logs' => $template->arrayRender($waf_logs),
					'waf_stats' => $template->arrayRender($waf_stats),
					'has_next_page' => $has_next_page,
					'service' => 'waf',
				];

				break;

			case 'cpu':
				WebTotemOption::setSessionOptions(['cpu_period' => $days]);

				$data = WebTotemAPI::getServerStatusData($host['id'], $days);
				$chart = WebTotem::generateChart($data['cpuChart'], $days);

				$_chart[] = [
					'variables' => [
						'days' => $days,
						'chart' => $chart,
					],
					'template' => 'cpu_chart',
				];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'service' => 'cpu',
				];

				break;

			case 'ram':
				WebTotemOption::setSessionOptions(['ram_period' => $days]);

				$data = WebTotemAPI::getServerStatusData($host['id'], $days);
				$chart = WebTotem::generateChart($data['ramChart'], $days);

				$_chart[] = [
					'variables' => [
						'days' => $days,
						'chart' => $chart,
					],
					'template' => 'ram_chart',
				];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'service' => 'ram',
				];

				break;

			case 'map':
				$data = WebTotemAPI::getFirewallChart($host['id'], $days);
				$chart = WebTotem::generateAttacksMapChart($data['map']);
				$world_map_json = WEBTOTEM_URL . '/includes/js/world_map.json';

				$_chart[] = [
					'variables' => [
						'attacks_map' => $chart,
						'world_map_json' => $world_map_json,
					],
					'template' => 'map_chart',
				];

				$response = [
					'chart' => $template->arrayRender($_chart),
					'service' => 'map',
				];

				break;

		}

		if ($service) {
			$response['success'] = true;
			$response['notifications'] = self::notifications();
			wp_send_json($response);
		}

	}

	/**
	 * Data lazy load.

	 * @return void
	 */
	public static function lazyLoad() {


		if (WebTotemRequest::post('ajax_action') !== 'lazy_load') {
			return;
		}

		$template = new WebTotemTemplate();

		$service = WebTotemRequest::post('service');

		$host = WebTotemAPI::siteInfo();

		switch ($service) {
			case 'all_sites':
				$cursor = WebTotemOption::getSessionOption('sites_cursor') ?: NULL;
				$allSites = WebTotemAPI::getSites($cursor);

				$has_next_page = $allSites['pageInfo']['hasNextPage'];

				WebTotemOption::setSessionOptions([
					'sites_cursor' => $allSites['pageInfo']['endCursor'],
				]);

				// Sites list.
				$build[] = [
					'variables' => [
						'sites' => WebTotem::allSitesData($allSites),
						'has_next_page' => $has_next_page,
					],
					'template' => 'multisite_list'
				];

				break;

			case 'firewall':
				$cursor = WebTotemOption::getSessionOption('firewall_cursor') ?: NULL;
				$period = WebTotemOption::getSessionOption('firewall_period') ?: 365;
				$data = WebTotemAPI::getFirewall($host['id'], 10, $cursor, $period);
				$service_data = $data['firewall'];
				$has_next_page = $service_data['logs']['pageInfo']['hasNextPage'];

				WebTotemOption::setSessionOptions([
					'firewall_cursor' => $service_data['logs']['pageInfo']['endCursor'],
				]);

				// Firewall logs.
				$build[] = [
					'variables' => [
						'logs' => WebTotem::wafLogs($service_data['logs']['edges']),
					],
					'template' => 'firewall_logs',
				];

				break;

			case 'antivirus':
				$cursor = WebTotemOption::getSessionOption('antivirus_cursor') ?: NULL;
				$event = WebTotemOption::getSessionOption('antivirus_event') ?: NULL;
				$permissions = WebTotemOption::getSessionOption('antivirus_permissions') ?: NULL;

				$params = [
					'host_id' => $host['id'],
					'limit' => 10,
					'days' => 365,
					'cursor' => $cursor,
					'event' => $event,
					'permissions' => $permissions,
				];

				$data = WebTotemAPI::getAntivirus($params);
				$has_next_page = $data['log']['pageInfo']['hasNextPage'];

				WebTotemOption::setSessionOptions([
					'antivirus_cursor' => $data['log']['pageInfo']['endCursor'],
				]);

				// Antivirus logs.
				$build[] = [
					'variables' => [
						'logs' =>WebTotem::getAntivirusLogs($data['log']['edges']),
					],
					'template' => 'antivirus_logs',
				];

				break;

			case 'reports':
				$cursor = WebTotemOption::getSessionOption('reports_cursor') ?: NULL;

				$data = WebTotemAPI::getAllReports($host['id'], 10, $cursor);
				$has_next_page = $data['pageInfo']['hasNextPage'];

				WebTotemOption::setSessionOptions([
					'reports_cursor' => $data['pageInfo']['endCursor'],
				]);

				// Reports.
				$build[] = [
					'variables' => [
						"reports" => WebTotem::getReports($data['edges']),
						"has_next_page" => $data['pageInfo']['hasNextPage'],
					],
					'template' => 'reports_list',
				];

				break;

			case 'reports_m':
				$cursor = WebTotemOption::getSessionOption('reports_m_cursor') ?: NULL;

				$data = WebTotemAPI::getAllReports($host['id'], 10, $cursor);
				$has_next_page = $data['pageInfo']['hasNextPage'];

				WebTotemOption::setSessionOptions([
					'reports_m_cursor' => $data['pageInfo']['endCursor'],
				]);

				// Reports mobile.
				$build[] = [
					'variables' => [
						"reports" => WebTotem::getReports($data['edges']),
						"has_next_page" => $data['pageInfo']['hasNextPage'],
					],
					'template' => 'reports_list_mobile',
				];

				break;
		}

		if ($service) {

			wp_send_json([
				'success' => true,
				'content' => $template->arrayRender($build),
				'has_next_page' => $has_next_page,
				'notifications' => self::notifications(),
			]);
		}
	}


	/**
	 * Add date filter.
	 *
	 * @return void
	 */
	public static function wafDateFilter() {

		if (WebTotemRequest::post('ajax_action') !== 'waf_date_filter') {
			return;
		}

		$template = new WebTotemTemplate();

		$date_from = WebTotemRequest::post('date_from');

		$period = explode(" to ", $date_from);
		WebTotemOption::setSessionOptions(['firewall_period' => $period]);

		$host = WebTotemAPI::siteInfo();

		// Firewall logs.
		$data = WebTotemAPI::getFirewall($host['id'], 10, NULL, $period);
		$firewall = $data['firewall'];

		$waf_logs[] = [
			'variables' => [
				'logs' => WebTotem::wafLogs($firewall['logs']['edges']),
			],
			'template' => 'firewall_logs',
		];

		// Firewall chart.
		$data = WebTotemAPI::getFirewallChart($host['id'], $period);
		$chart = WebTotem::generateWafChart($data['chart']);

		$_chart[] = [
			'variables' => [
				'days' => $chart['days'],
				'chart' => $chart['chart'],
			],
			'template' => 'firewall_chart',
		];

		// Firewall stats.
		$waf_stats[] = [
			'variables' => [
				'is_waf_training' => WebTotem::isWafTraining($data['agentManager']['createdAt']),
				'all_attacks' => $chart['count_attacks'],
				'blocking' => $chart['count_blocks'],
				'not_blocking' => $chart['count_attacks'] - $chart['count_blocks'],
				'most_attacks' => WebTotem::getMostAttacksData($firewall['map']),
			],
			'template' => 'firewall_stats',
		];

		WebTotemOption::setSessionOptions([
			'firewall_cursor' => $firewall['logs']['pageInfo']['endCursor'],
		]);

		$has_next_page = $firewall['logs']['pageInfo']['hasNextPage'];

		$response = [
			'success' => true,
			'chart' => $template->arrayRender($_chart),
			'waf_logs' => $template->arrayRender($waf_logs),
			'waf_stats' => $template->arrayRender($waf_stats),
			'has_next_page' => $has_next_page,
			'notifications' => self::notifications(),
		];

		wp_send_json($response);
	}


	/**
	 * Request to restart re-scan and receive antivirus data.
	 *
	 * @return void
	 */
	public static function antivirus() {

		if (WebTotemRequest::post('ajax_action') !== 'antivirus') {
			return;
		}

		$action = WebTotemRequest::post('av_action');

		$host = WebTotemAPI::siteInfo();

		switch ($action) {
			case 'rescan':
				$response = WebTotemAPI::forceCheck($host['id'], 'av');

				if (!isset($response['errors'])) {
					$data = WebTotemAPI::getAntivirusLastTest($host['id']);
					$response['last_scan'] = WebTotem::dateFormatter($data['lastTest']['time']);

				}
				break;

			case 'download_report':
				$response = WebTotemAPI::avExport($host['id']);
				if (!isset($response['errors'])) {
					$response['doc_link'] = $response['data']['auth']['sites']['av']['export'];
				}
				break;

			case 'filter':

				$file_status = WebTotemRequest::post('file_status');
				$permission = filter_var( WebTotemRequest::post('permission'), FILTER_VALIDATE_BOOLEAN);

				WebTotemOption::setSessionOptions([
					'antivirus_permissions' => $permission,
					'antivirus_event' => $file_status,
				]);

				$params = [
					'host_id' => $host['id'],
					'limit' => 10,
					'days' => 365,
					'cursor' => NULL,
					'event' => $file_status,
					'permissions' => $permission,
				];

				$data = WebTotemAPI::getAntivirus($params);
				$has_next_page = $data['log']['pageInfo']['hasNextPage'];

				WebTotemOption::setSessionOptions([
					'antivirus_cursor' => $data['log']['pageInfo']['endCursor'],
				]);

				// Antivirus logs.
				$build[] = [
					'variables' => [
						'logs' =>WebTotem::getAntivirusLogs($data['log']['edges']),
					],
					'template' => 'antivirus_logs',
				];

				$template = new WebTotemTemplate();
				$response = [
					'logs' => $template->arrayRender($build),
					'has_next_page' => $has_next_page,
				];

				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();

		wp_send_json($response);
	}

	/**
	 * Request to add a file to quarantine.
	 *
	 * @return void
	 */
	public static function quarantine() {
		if (WebTotemRequest::post('ajax_action') !== 'quarantine') {
			return;
		}

		$action = WebTotemRequest::post('quarantine_action');
		$id_or_path = WebTotemRequest::post('id_or_path');

		$host = WebTotemAPI::siteInfo();
		$response = [];

		switch ($action) {
			case 'add':
				$api_response = WebTotemAPI::moveToQuarantine($host['id'], $id_or_path);
				break;

			case 'remove':
				$api_response = WebTotemAPI::moveFromQuarantine($id_or_path);
				break;
		}

		if (!isset($api_response['errors'])) {

			$quarantine_logs = WebTotemAPI::getQuarantineList($host['id']);
			$quarantine_count = count($quarantine_logs);

			// Quarantine logs.
			$quarantine[] = [
				'variables' => [
					"logs" => WebTotem::getQuarantineLogs($quarantine_logs),
					"count" => $quarantine_count,
				],
				'template' => 'quarantine',
			];


			$cursor = WebTotemOption::getSessionOption('antivirus_cursor') ?: NULL;
			$event = WebTotemOption::getSessionOption('antivirus_event') ?: NULL;
			$permissions = WebTotemOption::getSessionOption('antivirus_permissions') ?: NULL;

			$params = [
				'host_id' => $host['id'],
				'limit' => 10,
				'days' => 365,
				'cursor' => $cursor,
				'event' => $event,
				'permissions' => $permissions,
			];

			$data = WebTotemAPI::getAntivirus($params);
			$has_next_page = $data['log']['pageInfo']['hasNextPage'];

			WebTotemOption::setSessionOptions([
				'antivirus_cursor' => $data['log']['pageInfo']['endCursor'],
			]);

			// Antivirus logs.
			$antivirus_logs[] = [
				'variables' => [
					'logs' =>WebTotem::getAntivirusLogs($data['log']['edges']),
				],
				'template' => 'antivirus_logs',
			];


			$template = new WebTotemTemplate();
			$response = [
				'antivirus_logs' => $template->arrayRender($antivirus_logs),
				'quarantine' => $template->arrayRender($quarantine),
				'has_next_page' => $has_next_page,
			];

		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();

		wp_send_json($response);

	}

	/**
	 * Request to add or remove a port to the ignore list.
	 *
	 * @return void
	 */
	public static function ignorePorts() {

		if (WebTotemRequest::post('ajax_action') !== 'ignore_ports') {
			return;
		}

		$template = new WebTotemTemplate();

		$action = WebTotemRequest::post('port_action');
		$port = (int) WebTotemRequest::post('port');

		$host = WebTotemAPI::siteInfo();

		switch ($action) {
			case 'add':
				$response = WebTotemAPI::addIgnorePort($host['id'], $port);
				break;

			case 'remove':
				$response = WebTotemAPI::removeIgnorePort($host['id'], $port);
				break;
		}

		if (!isset($response['errors'])) {

			$ports = WebTotemAPI::getAllPortsList($host['id']);
			$open_ports[] = [
				'variables' => [
					"ports" => $ports,
				],
				'template' => 'open_ports',
			];

			$ignore_ports[] = [
				'variables' => [
					"ports" => $ports,
				],
				'template' => 'ignore_ports',
			];
			$response = [
				'open_ports' => $template->arrayRender($open_ports),
				'ignore_ports' => $template->arrayRender($ignore_ports),
				'notifications' => self::notifications(),
			];

		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();

		wp_send_json($response);
	}

	/**
	 * Request for a report link.
	 *
	 * @return void
	 */
	public static function reports() {

		if (WebTotemRequest::post('ajax_action') !== 'reports') {
			return;
		}

		$template = new WebTotemTemplate();

		$action = WebTotemRequest::post('report_action');

		switch ($action) {
			case 'download':
				$id = WebTotemRequest::post('id');
				$link = WebTotemAPI::downloadReport($id);
				if ($link) {
					$response['link'] = $link;
				}
				break;
			case 'report_form':

				$period = explode(" to ", WebTotemRequest::post('date_period'));
				$modules_data = WebTotemRequest::post('modules');

				$modules = [
					'wa' => 'false',
					'dc' => 'false',
					'ps' => 'false',
					'rc' => 'false',
					'sc' => 'false',
					'av' => 'false',
					'waf' => 'false'
				];

				foreach ($modules_data as $module => $value){
					$modules[$module] = 'true';
				}

				$host = WebTotemAPI::siteInfo();
				$api_response = WebTotemAPI::generateReport($host['id'], $period, $modules);

				if (!$api_response) {
					$massage = '<div class="message error_message">' . __('Report generation error', 'wtotem') . '</div>';
				}
				else {
					$data = WebTotemAPI::getAllReports($host['id']);

					// Reports.
					$build[] = [
						'variables' => [
							"reports" => WebTotem::getReports($data['edges']),
							"has_next_page" => $data['pageInfo']['hasNextPage'],
						],
						'template' => 'reports_list',
					];

					// Reports mobile.
					$build_mobile[] = [
						'variables' => [
							"reports" => WebTotem::getReports($data['edges']),
							"has_next_page" => $data['pageInfo']['hasNextPage'],
						],
						'template' => 'reports_list_mobile',
					];

					$response = [
						'reports' => $template->arrayRender($build),
						'reports_m' => $template->arrayRender($build_mobile),
						'link' => $api_response,
					];

					$massage = '<div class="message success_message">' . __('The report was successfully generated', 'wtotem') . '</div>';
				}

				$response['massage'] = $massage;

				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();
		wp_send_json($response);
	}

	/**
	 * Request for a report link.
	 *
	 * @return void
	 */
	public static function settings() {

		if (WebTotemRequest::post('ajax_action') !== 'settings') {
			return;
		}

		$av_installed = WebTotemOption::getOption('av_installed');
		$waf_installed = WebTotemOption::getOption('waf_installed');

		if(!$av_installed && !$waf_installed) {
			WebTotemOption::setNotification('warning', __('It is not possible to make changes because the agents are not installed.', 'wtotem'));

			wp_send_json([
				'success' => false,
				'notifications' => self::notifications()
			]);
		}

		$action = WebTotemRequest::post('settings_action');
		$host = WebTotemAPI::siteInfo();
		$template = new WebTotemTemplate();

		switch ($action) {

			case 'module_toggle':
				$config = WebTotemAPI::toggleConfigs(WebTotemRequest::post('value'));
				$response['isActive'] = $config['isActive'];
				break;

			case 'module_notifications':
				$config = WebTotemAPI::toggleNotifications($host['id'], WebTotemRequest::post('value'));
				$response['isActive'] = $config;
				break;

			case 'waf_settings':

				$settings = [
					'gdn' => WebTotemRequest::post('gdn'),
					'dosProtection' => WebTotemRequest::post('dos'),
					'dosLimit' => WebTotemRequest::post('dos_limit'),
					'loginAttemptsProtection' => WebTotemRequest::post('login_attempt'),
					'loginAttemptsLimit' => WebTotemRequest::post('login_attempt_limit'),
				];

				$host = WebTotemAPI::siteInfo();
				$api_response = WebTotemAPI::setFirewallSettings($host['id'], $settings);

				if (!$api_response['errors']) {
					WebTotemOption::setNotification('success', __('Your changes have been applied successfully.', 'wtotem'));
				}

				break;

			case 'add_allow_ip':
        $api_response = WebTotemAPI::addIpToList($host['id'], WebTotemRequest::post('value'), 'white');
				if ($api_response) {
					$data = WebTotemAPI::getIpLists($host['id']);
					$build[] = [
						'variables' => [
							"list" => WebTotem::getIpList($data['whiteList'], 'ip_allow'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
				}
				break;

			case 'add_deny_ip':
				$api_response = WebTotemAPI::addIpToList($host['id'], WebTotemRequest::post('value'), 'black');
				if ($api_response) {
					$data = WebTotemAPI::getIpLists($host['id']);
					$build[] = [
						'variables' => [
							"list" => WebTotem::getIpList($data['blackList'], 'ip_deny'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
				}
				break;

			case 'add_allow_url':
				$api_response = WebTotemAPI::addUrlToAllowList($host['id'], WebTotemRequest::post('value'));
				if ($api_response) {
					$data = WebTotemAPI::getAllowUrlList($host['id']);
					$build[] = [
						'variables' => [
							"list" => WebTotem::getUrlAllowList($data),
						],
						'template' => 'allow_url_list',
					];

					$response['content'] = $template->arrayRender($build);
				}
				break;

			case 'add_ip_list':
				$ips = WebTotemRequest::post('ips');
				$list_name = WebTotemRequest::post('list');

				$host = WebTotemAPI::siteInfo();
				$api_response = WebTotemAPI::addIpToList($host['id'], $ips, $list_name);

				if ($api_response) {
					$data = WebTotemAPI::getIpLists($host['id']);

					$data_list = ($list_name == 'white') ? $data['whiteList'] : $data['blackList'];
					$ip_list = ($list_name == 'white') ? 'ip_allow' : 'ip_deny';

					$build[] = [
						'variables' => [
							"list" => WebTotem::getIpList($data_list, $ip_list),
						],
						'template' => 'allow_deny_list',
					];

					if ($api_response['status'] != 0) {
						$response['invalidIPs'] = implode("\n", $api_response['invalidIPs']);
					}

					$response['wrap'] = ($list_name == 'white') ? '#wtotem_ip_allow_list' : '#wtotem_ip_deny_list';
					$response['content'] = $template->arrayRender($build);
				}
				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();
		wp_send_json($response);
	}

	/**
	 * Request to remove from the list of deny/allowed ip or url addresses.
	 *
	 * @return void
	 */
	public static function remove() {

		if (WebTotemRequest::post('ajax_action') !== 'remove') {
			return;
		}

		$av_installed = WebTotemOption::getOption('av_installed');
		$waf_installed = WebTotemOption::getOption('waf_installed');

		if(!$av_installed && !$waf_installed) {
			WebTotemOption::setNotification('warning', __('It is not possible to make changes because the agents are not installed.', 'wtotem'));

			wp_send_json([
				'success' => false,
				'notifications' => self::notifications()
			]);
		}

		$action = WebTotemRequest::post('remove_action');
		$host = WebTotemAPI::siteInfo();
		$template = new WebTotemTemplate();

		switch ($action) {
			case 'ip_allow':
				$api_response = WebTotemAPI::removeIpFromList( WebTotemRequest::post('id') );

				if ($api_response) {
					$data = WebTotemAPI::getIpLists($host['id']);

					$build[] = [
						'variables' => [
							"list" => WebTotem::getIpList($data['whiteList'], 'ip_allow'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
					$response['wrap'] = '#wtotem_ip_allow_list';
				}
				break;

			case 'ip_deny':
				$api_response = WebTotemAPI::removeIpFromList( WebTotemRequest::post('id') );

				if ($api_response) {
					$data = WebTotemAPI::getIpLists($host['id']);

					$build[] = [
						'variables' => [
							"list" => WebTotem::getIpList($data['blackList'], 'ip_deny'),
						],
						'template' => 'allow_deny_list',
					];

					$response['content'] = $template->arrayRender($build);
					$response['wrap'] = '#wtotem_ip_deny_list';
				}
				break;

			case 'url_allow':
				$api_response = WebTotemAPI::removeUrlFromAllowList( WebTotemRequest::post('id') );

				if ($api_response) {
					$data = WebTotemAPI::getAllowUrlList($host['id']);

					$build[] = [
						'variables' => [
							"list" =>  WebTotem::getUrlAllowList($data),
						],
						'template' => 'allow_url_list',
					];

					$response['content'] = $template->arrayRender($build);
					$response['wrap'] = '#wtotem_allow_url';
				}
				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();
		wp_send_json($response);
	}

	/**
	 * Request to remove site from WebTotem.
	 *
	 * @return void
	 */
	public static function multisite() {

		if (WebTotemRequest::post('ajax_action') !== 'multisite') {
			return;
		}

		$action = WebTotemRequest::post('multisite_action');
		$template = new WebTotemTemplate();

		switch ($action) {
			case 'remove_site':

				$host_id = WebTotemRequest::post('hid');
				$main_host = WebTotemOption::getMainHost();

				if($host_id == $main_host['id']){
					WebTotemOption::setNotification('error', __('You cannot delete the primary domain.', 'wtotem'));
					break;
				}
				WebTotemAPI::removeMultiSiteHost($host_id);

				break;

			case 'add_site':

				$new_site = WebTotemRequest::post('site_name');
				WebTotemAPI::addMultiSiteNewSites([$new_site]);

				break;
		}

		$allSites = WebTotemAPI::getSites();
		$has_next_page = $allSites['pageInfo']['hasNextPage'];

		WebTotemOption::setSessionOptions([
			'sites_cursor' => $allSites['pageInfo']['endCursor'],
		]);

		// Sites list.
		$build[] = [
			'variables' => [
				'sites' => WebTotem::allSitesData($allSites),
				'has_next_page' => $has_next_page,
			],
			'template' => 'multisite_list'
		];

		$response['content'] = $template->arrayRender($build);

		$response['success'] = true;
		$response['notifications'] = self::notifications();
		wp_send_json($response);
	}

	/**
	 * Changing the theme mode.
	 *
	 * @return void
	 */
	public static function changeThemeMode() {

		if (WebTotemRequest::post('ajax_action') !== 'theme_mode') {
			return;
		}

		$theme_mode = WebTotemOption::getSessionOption('theme_mode');

		if ($theme_mode == 'dark') {
			WebTotemOption::setSessionOptions(['theme_mode' => 'light']);
			$response = 'light';
		}
		else {
			WebTotemOption::setSessionOptions(['theme_mode' => 'dark']);
			$response = 'dark';
		}

		wp_send_json($response);
	}

	/**
	 * Set user time zone offset.
	 *
	 * @return void
	 */
	public static function userTimeZone() {

		if (WebTotemRequest::post('ajax_action') !== 'set_time_zone') {
			return;
		}

		$time_zone_offset = WebTotemRequest::post('offset');
		$now = strtotime('now');
		$check = WebTotemOption::getOption('time_zone_check') ?: 0;

		// Checking whether an hour has elapsed since the previous request.
		if ($now >= $check) {
			$time_zone = WebTotemAPI::getTimeZone();
			if ($time_zone) {
				$time_zone_offset = timezone_offset_get(new \DateTimeZone($time_zone), new \DateTime('now', new \DateTimeZone('Europe/London'))) / 3600;
				WebTotemOption::setOptions(['time_zone_check' => $now + 3600]);
			}
			WebTotemOption::setOptions(['time_zone_offset' => $time_zone_offset]);
		}

		wp_send_json([
			'success' => true,
			'time_zone_offset' => $time_zone_offset
		]);

	}

	/**
	 * Updating the page data in the specified time interval.
	 *
	 * @return void
	 */
	public static function reloadPage() {

		if (WebTotemRequest::post('ajax_action') !== 'reload_page') {
			return;
		}

		$page = WebTotemRequest::post('page');

		$template = new WebTotemTemplate();

		// Get data from WebTotem API.
		$host = WebTotemAPI::siteInfo();

		switch ($page) {
			case 'dashboard':

				$data = WebTotemAPI::getAllData($host['id']);

				// Start build array for rendering.
				// Scoring block.
				$service_data = $data['scoring']['result'];
				$total_score = round($data['scoring']['score']);
				$score_grading = WebTotem::scoreGrading($total_score);
				$build['scoring'] = [
					'variables' => [
						"host_id" => $host['id'],
						"total_score" => $total_score . "%",
						"tested_on" => $data['scoring']['lastTest']['time'],
						"server_ip" => $service_data['ip'] ?: ' - ',
						"location" => WebTotem::getCountryName($service_data['country']) ?: ' - ',
						"is_higher_than"  => $service_data['isHigherThan'] . '%',
						"grade" => $score_grading['grade'],
						"color" => $score_grading['color'],
					],
					'template' => 'score',
				];

				// Firewall stats.
				$period = WebTotemOption::getSessionOption('firewall_period');
				$service_data = $period ? WebTotemAPI::getFirewall($host['id'], 10, NULL, $period) : $data;
				$service_data = $service_data['firewall'];

				$chart = WebTotem::generateWafChart($service_data['chart']);
				$build['firewall_stats'] = [
					'variables' => [
						"is_waf_training" => $data['agentManager'] && WebTotem::isWafTraining( $data['agentManager']['createdAt'] ),
						"most_attacks" => WebTotem::getMostAttacksData($service_data['map']),
						"all_attacks" => $chart['count_attacks'],
						"blocking" => $chart['count_blocks'],
						"not_blocking" => (int) $chart['count_attacks'] - (int) $chart['count_blocks'],
					],
					'template' => 'firewall_stats',
				];

				$build['chart_periods'] = [
					'variables' => [
						"service" => 'waf',
						"days" => is_array($period) ? 7 : $period,
					],
					'template' => 'chart_periods',
				];

				// Firewall blocks.
				$build['firewall_data'] = [
					'variables' => [
						"chart" => $chart['chart'],
						"days" => $chart['days'],
						"logs"  => WebTotem::wafLogs($service_data['logs']['edges']),
					],
					'template' => 'firewall',
				];

				// Server Status RAM.
				$period = WebTotemOption::getSessionOption('ram_period') ?: 7;
				$service_data = $period ? WebTotemAPI::getServerStatusData($host['id'], $period) : $data['serverStatus'];

				$build['server_status_ram'] = [
					'variables' => [
						"info" => $service_data['info'],
						"ram_chart" => WebTotem::generateChart($service_data['ramChart']),
						"days" => $period,
					],
					'template' => 'server_status_ram',
				];

				// Server Status CPU.
				$period = WebTotemOption::getSessionOption('cpu_period') ?: 7;
				$service_data = $period ? WebTotemAPI::getServerStatusData($host['id'], $period) : $data['serverStatus'];
				$build['server_status_cpu'] = [
					'variables' => [
						"cpu_chart" => WebTotem::generateChart($service_data['cpuChart']),
						"days" => $period,
					],

					'template' => 'server_status_cpu',
				];

				// Antivirus stats blocks.
				$antivirus_stats = $data['antivirus']['stats'];
				$build['antivirus_stats'] = [
					'variables' => [
						"changes"  => $antivirus_stats['changed'] ?: 0,
						"scanned"  => $antivirus_stats['scanned'] ?: 0,
						"deleted"  => $antivirus_stats['deleted'] ?: 0,
						"infected" => $antivirus_stats["infected"] ?: 0,
					],

					'template' => 'antivirus_stats',
				];

				// Monitoring blocks.
				$build['monitoring'] = [
					'variables' => [
						"ssl"  => [
							'status' => WebTotem::getStatusData($data['ssl']['status']),
							'days_left' => WebTotem::daysLeft($data['ssl']['expiryDate']),
							'issue_date' => WebTotem::dateFormatter($data['ssl']['issueDate']),
							'expiry_date' => WebTotem::dateFormatter($data['ssl']['expiryDate']),
						],
						"availability"  => [
							'status' => WebTotem::getStatusData($data['availability']['status']),
							"percent" => $data['availability']['percent'],
							"response_time" => ceil($data['availability']['responseTime'] / 1000000) . ' ' . __('ms.', 'wtotem'),
							"downtime" => ceil($data['availability']['downTime'] / 1000000) . ' ' . __('ms.', 'wtotem'),
							"last_test" => WebTotem::dateFormatter($data['availability']['lastTest']['time']),
						],
						'reputation' => [
							"status" => WebTotem::getStatusData($data['reputation']['status']),
							"blacklists_entries" => WebTotem::blacklistsEntries(
								$data['reputation']['status'],
								$data['reputation']['virusList']),
							"info" => WebTotem::getReputationInfo($data['reputation']['status']),
							"last_test" => WebTotem::dateFormatter($data['reputation']['lastTest']['time']),
						],
					],
					'template' => 'monitoring',
				];


				// Scanning blocks.
				$disc_usage_data = $data['serverStatus']['discUsage'];
				$disc_usage = [
					'total' => $disc_usage_data['total'],
					'free'  => $disc_usage_data['free'],
					'used'  => $disc_usage_data['total'] - $disc_usage_data['free'],
				];

				$build['scanning'] = [
					'variables' => [
						"ports"  => [
							'status' => WebTotem::getStatusData($data['ports']['status']),
							"ip" => $data['ports']['ip'],
							"number" => count($data['ports']['tcp']),
							"tcp" => $data['ports']['tcp'],
							"ignore_ports" => $data['ports']['ignorePorts'],
							"last_test" => WebTotem::dateFormatter($data['ports']['lastTest']['time']),
						],
						"deface"  => [
							'status' => WebTotem::getStatusData($data['deface']['status']),
							"number" => $data['deface']['count'],
							"words" => !empty($data['deface']['words']) ? implode(",", $data['deface']['words']) : '',
							"last_test" => WebTotem::dateFormatter($data['deface']['lastTest']['time']),
						],
						"disc_usage" => $disc_usage,
						"disc_chart" => json_encode($disc_usage),
					],
					'template' => 'scanning',
				];

				$response['content'][] = ['selector' => '#scoring', 'content' => $template->arrayRender($build['scoring'])];
				$response['content'][] = ['selector' => '#firewall_stats', 'content' => $template->arrayRender($build['firewall_stats'])];
				$response['content'][] = ['selector' => '#waf_chart_period', 'content' => $template->arrayRender($build['chart_periods'])];
				$response['content'][] = ['selector' => '#firewall_data', 'content' => $template->arrayRender($build['firewall_data'])];
				$response['content'][] = ['selector' => '#server_status_cpu', 'content' => $template->arrayRender($build['server_status_cpu'])];
				$response['content'][] = ['selector' => '#server_status_ram', 'content' => $template->arrayRender($build['server_status_ram'])];
				$response['content'][] = ['selector' => '#antivirus_stats', 'content' => $template->arrayRender($build['antivirus_stats'])];
				$response['content'][] = ['selector' => '#monitoring', 'content' => $template->arrayRender($build['monitoring'])];
				$response['content'][] = ['selector' => '#scanning', 'content' => $template->arrayRender($build['scanning'])];

				break;
		}

		$response['success'] = true;
		$response['notifications'] = self::notifications();
		wp_send_json($response);
	}

	/**
	 * Notification output.
	 *
	 * @return string
	 */
	public static function notifications() {

		$notifications = WebTotem::getNotifications();

		if($notifications){
			$build[] = [
				'variables' => [
					'notifications' => $notifications,
				],

				'template' => 'notifications',
			];

			$template = new WebTotemTemplate();
			return $template->arrayRender($build);
		}
		return false;


	}

}
